#include "../include/aprecv-general.h"

void clean_exit (int sig) {

  /* Print captured TCP stream */
  if(print_tcp_stream && !no_dissection) 
   	  print_tcp_streams();

  /* Print statistic counter */  
  if(statistic && !no_dissection) 
	  print_statistic();
  
  /* Show Libpcap capture statistic */
  if(pcap_log_file == NULL && !use_apsrlib) 
	  cap_stats();
  else if(!use_apsrlib)
	  apsr_output("\nRead pcap dump file completed.\n");

  /* Close Libpcap dump file */
  if(aprecv_log_raw_file && !use_apsrlib) 
	  pcap_dump_close(dump);

  /* Aprecv text log file close */
  if(aprecv_log_file) { 
	  if(fclose(stdout) < 0) {
		  apsr_stream_output((FILE *) stderr, "Error: Can't close logfile %s.\n", aprecv_log_file);
		  #ifdef APRECV_DEBUG
		  apsr_stream_output((FILE *) stderr, "Error code: %s.\n", 
				  strerror(errno));
		  #endif
	  }
  }

  /* Iptrace file close */
  if(iptrace_trace_file) {
	  #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
	  if(gzclose(file_handle) < 0) {
	  #else
	  if(fclose(file_handle) < 0) {
	  #endif
		  apsr_stream_output((FILE *) stderr, "Error: Can't close tracefile %s.\n", iptrace_trace_file);
		  #ifdef APRECV_DEBUG
		  apsr_stream_output((FILE *) stderr, "Error code: %s.\n", strerror(errno));
		  #endif
          }
  }

  /* Snoop file close */	  
  if(snoop_trace_file) {
	  #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
	  if(gzclose(file_handle) < 0) {
          #else
          if(fclose(file_handle) < 0) {
          #endif
		  apsr_stream_output((FILE *) stderr, "Error: Can't close tracefile %s.\n", snoop_trace_file);
		  #ifdef APRECV_DEBUG
		  apsr_stream_output((FILE *) stderr, "Error code: %s.\n", strerror(errno));
		  #endif
          }
  }
  
  /* Remove .pid file */
  if(aprecv_daemon)
	  unlink("/var/run/aprecv.pid");

  #ifdef HAVE_APSRLIB_MODULE_SUPPORT
  close_modules();
  #endif

  /* Close connection to apsrserver */
  if(server_addr)
	  close_server_connection();

  #ifdef HAVE_APSRLIB_BLUETOOTH_SUPPORT
  if(!use_apsrlib && !use_apsrlib_bluetooth)
  #else
  if(!use_apsrlib)	  
  #endif	 
	  /* Close socket file descriptor for Libpcap */
	  pcap_close(recv_d);  
  #ifdef HAVE_APSRLIB_BLUETOOTH_SUPPORT
  else if(use_apsrlib_bluetooth)
	  /* Close socket file descriptor for Bluetooth */
	  apsr_close_bluetooth_socket(apsrlib_bluetooth_fd);
  #endif
  
  apsr_output("\nExit");
  #ifdef APRECV_DEBUG
  switch(sig) {
	  case SIGTERM:
	    apsr_output(" (Code: Term - 15)");
	    break;
	  case SIGQUIT:
	    apsr_output(" (Code: Quit - 3)");
            break;
          case SIGINT:
 	    apsr_output(" (Code: Interrupt - 2)");
            break;
          case SIGABRT:
            apsr_output(" (Code: Abort - 6)");
            break;
          case SIGSEGV:
            apsr_output(" (Code: Segfault - 11)");
            break;
	  default:
	    apsr_output(" (Code: %d)", sig);
	    break;
  }
  #endif
  apsr_output(".\n");
  
  exit(EXIT_SUCCESS);	
}
