#include "../include/aprecv-general.h"

static const char *eigrpopcodes[]= {
	" (Unknown)",
	" (Update)",
	" (Request)",
	" (Query)",
	" (Reply)",
	" (Hello)",
	" (IPX SAP)",
};

int print_eigrp_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct eigrp_pkt_header *eigrp_header;
  static unsigned int flags;
  static unsigned short pkt_checksum, correct_checksum;
  
  eigrp_header = (struct eigrp_pkt_header *)(packet+data_link_size+opt_size);
  flags = apsr_ntohl(eigrp_header->flags);
  
  apsr_output("- EIGRP Header -\n  Version: %d, Opcode: %d", eigrp_header->version, eigrp_header->opcode);
  if(verbose)
	apsr_output("%s", eigrpopcodes[eigrp_header->opcode]);

  pkt_checksum = apsr_ntohs(eigrp_header->sum);
  apsr_output("\n  Checksum: 0x%04x", pkt_checksum);

  if(calc_checksums) {
    eigrp_header->sum=0x0000;
    correct_checksum=apsr_ntohs(in_cksum((unsigned short *)eigrp_header,
			    (header->len-data_link_size-opt_size), 0));
    if(pkt_checksum == correct_checksum) {
	    apsr_output(" (correct checksum)");
    }
    else {
	    apsr_output(" (incorrect checksum - should be 0x%04x)",
			    correct_checksum);

            if(statistic)
                          stats.cr.eigrp++;
    }
  }
    
  apsr_output("\n  Flags: 0x%08x", eigrp_header->flags);
  if(verbose) {
	  if(eigrp_header->flags & APSR_EIGRP_FLAGS_INIT) 
		  apsr_output(" (Init bit)");
          if(eigrp_header->flags & APSR_EIGRP_FLAGS_CONDITIONAL) 
		  apsr_output(" (Conditional receive bit)");
  }
  apsr_output("\n  Sequence Number: %u, Ack Seq. Number: %u\n", 
		  apsr_ntohl(eigrp_header->seq_num), apsr_ntohl(eigrp_header->ack_num));
  apsr_output("  Autosys: %u\n", apsr_ntohl(eigrp_header->autosys));
  
  if(statistic) 
          stats.c.eigrp++;
  
  return(RET_SUCCESS);
}
