#include "../include/aprecv-general.h"

int print_fddi_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct fddi_pkt_header *fddi_header;

  fddi_header = (struct fddi_pkt_header *) packet;
  
  apsr_output("\n- FDDI Header -");

  if(header->caplen < APSR_FDDI_H) {
	  apsr_output("\nError: Captured data length(%d bytes) < FDDI packet header length.\n", (int) header->caplen);
	  if(statistic)
		  stats.tr.fddi++;
	  goto end_fddi;
  }

  if(expand_fddi || expand_all) {
    apsr_output("\n  Frame Control: 0x%02x", fddi_header->fc);
    
  if(verbose) {
  	switch(fddi_header->fc) {
		case APSR_FDDIFC_VOID:
			apsr_output(" (void frame)");
			break;
		case APSR_FDDIFC_NRT:
			apsr_output(" (non-restricted token)");
			break;
		case APSR_FDDIFC_RT:
			apsr_output(" (restricted token)");
			break;
		case APSR_FDDIFC_SMT_INFO:
			apsr_output(" (SMT info)");
			break;
		case APSR_FDDIFC_SMT_NSA:
			apsr_output(" (SMT next station address)");
			break;
		case APSR_FDDIFC_MAC_BEACON:
			apsr_output(" (MAC beacon frame)");
			break;
		case APSR_FDDIFC_MAC_CLAIM:
			apsr_output(" (MAC claim frame)");
			break;
			
		default:
			switch(fddi_header->fc & APSR_FDDIFC_CLFF) {
				case APSR_FDDIFC_MAC:
					apsr_output(" (mac%1x)", fddi_header->fc & APSR_FDDIFC_ZZZZ);
					break;
				case APSR_FDDIFC_SMT:
					apsr_output(" (smt%1x)", fddi_header->fc & APSR_FDDIFC_ZZZZ);
					break;
				case APSR_FDDIFC_LLC_ASYNC:
					apsr_output(" (async%1x)", fddi_header->fc & APSR_FDDIFC_ZZZZ);
					break;
				case APSR_FDDIFC_LLC_SYNC:
					apsr_output(" (sync%1x)", fddi_header->fc & APSR_FDDIFC_ZZZZ);
					break;
				case APSR_FDDIFC_IMP_ASYNC:
					apsr_output(" (imp async%1x)", fddi_header->fc & APSR_FDDIFC_ZZZZ);
					break;
				case APSR_FDDIFC_IMP_SYNC:
					apsr_output(" (imp sync%1x)", fddi_header->fc & APSR_FDDIFC_ZZZZ);
					break;
			}
		}
	}

    apsr_output("\n  MAC Dst: %s", ether2str(fddi_header->destination));
    if(verbose)
         resolve_mac(fddi_header->destination);
	 
    apsr_output("\n  MAC Src: %s", ether2str(fddi_header->source));
    if(verbose)
         resolve_mac(fddi_header->source);
  }

  if(statistic) 
  	  stats.c.fddi++;

end_fddi:
  apsr_output("\n");

  return(RET_SUCCESS);
}
