#include <apsrlib/apsrlib.h>

int test(struct sockaddr *, unsigned char *, unsigned int);

int main (void) {
	struct apsr_link_layer *apsr_ll;
	char *device="eth0";
	
	/* Open a link layer interface. */
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

	apsr_read_link_layer(apsr_ll, 1500, 0, 
			(apsr_sniff_callback)test);
	
	/* Close link layer */
	apsr_close_link_layer(apsr_ll);
	
	return(RET_SUCCESS);
}

int test(struct sockaddr *sa, unsigned char *buf, unsigned int len) {
  int i=0, x, count=0;
  struct ipv4_pkt_header *ipv4_header;
  static unsigned short fragment, length, ident, version, header_length, type_service, pkt_checksum;
  static char *source_ip, *dest_ip;
  
  ipv4_header = (struct ipv4_pkt_header *)(buf+14);
  
  version       = APSR_IPV4_VERSION(ipv4_header);
  header_length = APSR_IPV4_HEADERLENGTH(ipv4_header);
  fragment      = ntohs(ipv4_header->fragment);
  type_service  = ntohs(ipv4_header->type_service);
  length        = ntohs(ipv4_header->packet_len);
  ident         = ntohs(ipv4_header->ident);
  source_ip     = strdup(inet_ntoa(ipv4_header->src_ip));
  dest_ip       = strdup(inet_ntoa(ipv4_header->dst_ip));
  pkt_checksum = ntohs(ipv4_header->sum);
    

  apsr_output("\n- IPv4 Header -\n  Version: %d, Hdr-Len: %d, Length: %d\n", version, header_length, length);
  apsr_output("  TTL: %d, TOS: 0x%02x", ipv4_header->time_live, type_service);
  apsr_output(", ID: %u\n  Frag.: 0x%x", ident, fragment);
  apsr_output("\n  Src IP: %s", source_ip);
  apsr_output("\n  Dst IP: %s", dest_ip);
  apsr_output("\n  Protocol: %d", ipv4_header->protocol);
  apsr_output("\n  Checksum: 0x%04x\n", pkt_checksum);
  
  for(x = 0; x <= len; x++) {
	if((i++ % 12) == 0 || x == 0) {
		count++;
		apsr_output("\n0x%04x:", count);
	}
	
    	apsr_output("%02x", buf[i]);
  }
  apsr_output("\n");
  
  return(RET_SUCCESS);
}

