#include <stdio.h>
#include <string.h>
#include <apsrlib/apsrlib.h>

int main (void) {
	struct apsr_link_layer *apsr_ll;
	char *device="lo";
	unsigned char src_mac[6] = {0x0d, 0x0e, 0x0a, 0x0d, 0x00, 0x00};
	unsigned char dst_mac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
	struct in6_addr src_ip, dst_ip;

	
	int my_ret, written;
	unsigned char *buf;

	/* Open a link layer interface. */
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

	/* Initialize the packet memory. */
	/* We need 14 bytes for the ethernet header and */
	/* 40 bytes for the IPv6 header. */
	my_ret = apsr_init_psize(APSR_ETH_H + APSR_IPv6_H, &buf);

	if(my_ret == 0 || buf == NULL) ERROR("apsr_init_psize failed!");

	/* Build test packet */
	/* Firstly we construct the ethernet header. */
	my_ret = apsr_ethernet(dst_mac,		// Destination address
		     	       src_mac,		// Source address
			       ETHERTYPE_IP6,	// Ethernet type
		   	       NULL,		// Payload
			       0,		// Payload size
			       buf);		// Buffer

	if(my_ret == -1) ERROR("apsr_ethernet failed!");

        inet_pton(AF_INET6, "0000:0000:0000:0000:0000:0000:0000:0001", &src_ip);
	inet_pton(AF_INET6, "0000:0000:0000:0000:0000:0000:0000:0002", &dst_ip);
	
 	my_ret = apsr_ipv6(6,			// IP version
			   1,			// IP priority
			   0,			// IP flow label
			   42,			// IP payload length
			   19,			// IP next header
			   255,			// IP hop limit
			   src_ip,		// Source IP
			   dst_ip,		// Destination IP
			   NULL,		// IP payload
			   0,			// IP payload length
			   buf + APSR_ETH_H);	// Packet buffer

	if(my_ret == -1) ERROR("apsr_ipv6 failed!");

	/* Send it */
	written = apsr_write_link_layer(apsr_ll, device, buf,
					APSR_ETH_H + APSR_IPv6_H, 1);

	apsr_output("Wrote %i bytes!\n", written);
	
	/* Close link layer */
	apsr_close_link_layer(apsr_ll);
	
	return(RET_SUCCESS);
}

