#include <stdio.h>
#include <string.h>
#include <apsrlib/apsrlib.h>

int main (void) {
	struct apsr_link_layer *apsr_ll;
	char *device="lo";
	unsigned char src_mac[6] = {0x0d, 0x0e, 0x0a, 0x0d, 0x00, 0x00};
	unsigned char dst_mac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

	int my_ret, written;
	unsigned char *buf;

	/* Open a link layer interface. */
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

	/* Initialize the packet memory. */
	/* We need 14 bytes for the ethernet header, 20 bytes */
	/* for the IPv4 header and 6 bytes for the SCTP header. */
	my_ret = apsr_init_psize(APSR_ETH_H + APSR_IPv4_H + APSR_SCTP_H, &buf);

	if(my_ret == 0 || buf == NULL) ERROR("apsr_init_psize failed!");

	/* Build test packet */
	/* Firstly we construct the ethernet header. */
	my_ret = apsr_ethernet(dst_mac,		// Destination address
		     	       src_mac,		// Source address
			       ETHERTYPE_IP,	// Ethernet type
		   	       NULL,		// Payload
			       0,		// Payload size
			       buf);		// Buffer

	if(my_ret == -1) ERROR("apsr_ethernet failed!");

 	my_ret = apsr_ipv4(4,			// IP version
			   5,			// IP header length
			   20,			// IP length
			   0x1111,		// IP Type of Service (ToS)
			   45360,		// IP Identification (ID)
			   0x4000,		// IP Fragementation offset
			   64,			// IP Time to live (TTL)
			   APSR_PROTO_SCTP,	// IP transport layer protocol
			   inet_addr("127.0.0.1"), // Source IP
			   inet_addr("127.0.0.2"), // Destination IP
			   0,			// IP checksum
			   NULL,		// IP options
			   0,			// IP option size
			   NULL,		// IP payload
			   0,			// IP payload length
			   buf + APSR_ETH_H);	// Packet buffer

	if(my_ret == -1) ERROR("apsr_ipv4 failed!");

	my_ret = apsr_sctp(100,			// SCTP source port
			   100,			// SCTP destination port
			   0xFFFF,		// SCTP V_TAG
			   0x0001,		// SCTP checksum
			   NULL,		// SCTP payload
			   0,			// SCTP payload length
			   buf + APSR_ETH_H
			   + APSR_IPv4_H);	// Packet buffer

	/* Calculate the checksum */
	
	apsr_checksum(buf+APSR_ETH_H, APSR_PROTO_SCTP, APSR_SCTP_H);

	/* Send it */
	written = apsr_write_link_layer(apsr_ll, device, buf,
					APSR_ETH_H + APSR_IPv4_H + APSR_SCTP_H, 1);

	apsr_output("Wrote %i bytes!\n", written);
	
	/* Close link layer */
	apsr_close_link_layer(apsr_ll);
	
	return(RET_SUCCESS);
}

