#include <apsrlib/apsrlib.h>

#define MODULE_VERSION          "0.01"
#define MODULE_NAME             "madcap"
#define MODULE_DESCRIPTION      "Multicast Address Dynamic Client Allocation Protocol (MADCAP) sniffing module"
#define MODULE_IDENT_FAMILY     AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	2535
#define MODULE_IDENT_ID2        2535

#define MODULE_FILTER		"udp and port 2535"

static const char *mod_madcap_msgtype[]= {
	" (Reserved)",
	" (Discover)",
	" (Offer)",
	" (Request)",
	" (Renew)",
	" (Ack)",
	" (Nak)",
	" (Release)",
	" (Getinfo)",
};

struct mod_madcap_header {
	unsigned char 	version;
	unsigned char 	msg_type;
	unsigned short 	addr_family;
	unsigned int	xid;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version     = MODULE_VERSION;
	module_info.name        = MODULE_NAME;
	module_info.desc        = MODULE_DESCRIPTION;
	module_info.bpf_filter  = MODULE_FILTER;
  	module_info.family      = MODULE_IDENT_FAMILY;
    	module_info.proto1      = MODULE_IDENT_PN1;
      	module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
	      
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,                 unsigned short id1, unsigned short id2)
{
  struct mod_madcap_header *madcap_header;
	
  if(MODULE_IDENT_FAMILY == family && MODULE_IDENT_PN1 == protocol_number && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	madcap_header = (struct mod_madcap_header *) buf;

	apsr_output("- MADCAP header -\n  Version: %u", madcap_header->version);
	apsr_output(", Message : %u", madcap_header->msg_type);
	if(verbose)
		apsr_output("%s", mod_madcap_msgtype[madcap_header->msg_type]);
  	apsr_output("\n  Address Family: %u, Transaction ID: %u\n", apsr_ntohs(madcap_header->addr_family), apsr_ntohl(madcap_header->xid));
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
		  family, protocol_number, id1, id2);
  #endif

  return(RET_SUCCESS);
}

