#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.03"
#define MODULE_NAME		"pptp"
#define MODULE_DESCRIPTION	"Example PPTP (Point to Point Tunneling Protocol) protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_ID1	1723	
#define MODULE_IDENT_ID2	1723

#define MODULE_FILTER		"tcp and port 1723"

static const char *mod_pptp_messagetype[]= {
	" (Unknown)",
	" (Control)",
	" (Management)",
};

static const char *mod_pptp_cmt[]= {
	" (Unknown)",
	" (Start-Control-Connection-Request)",
	" (Start-Control-Connection-Reply)",
	" (Stop-Control-Connection-Request)",
	" (Stop-Control-Connection-Reply)",
	" (Echo-Request)",
	" (Echo-Reply)",
	" (Outgoing-Call-Request)",
	" (Outgoing-Call-Reply)",
	" (Incoming-Call-Request)",
	" (Incoming-Call-Reply)",
	" (Incoming-Call-Connected)",
	" (Call-Clear-Request)",
	" (Call-Disconnect-Notify)",
	" (WAN-Error-Notify)",
	" (Set-Link-Info)",
};
	
#define APSR_PPTP_MESSAGE_TYPE_CONTROL		1
#define APSR_PPTP_MESSAGE_TYPE_MANAGEMENT	2

#define APSR_PPTP_CMT_START_CC_REQUEST		1
#define APSR_PPTP_CMT_START_CC_REPLY		2
#define APSR_PPTP_CMT_STOP_CC_REQUEST		3
#define APSR_PPTP_CMT_STOP_CC_REPLY		4
#define APSR_PPTP_CMT_ECHO_REQUEST		5
#define APSR_PPTP_CMT_ECHO_REPLY		6
#define APSR_PPTP_CMT_OUTGOING_CALL_REQ		7
#define APSR_PPTP_CMT_OUTGOING_CALL_REP		8
#define APSR_PPTP_CMT_INCOMING_CALL_REQ		9
#define APSR_PPTP_CMT_INCOMING_CALL_REP		10
#define APSR_PPTP_CMT_INCOMING_CALL_CON		11
#define APSR_PPTP_CMT_CALL_CLEAR_REQ		12
#define APSR_PPTP_CMT_CALL_DISCO_NOT		13
#define APSR_PPTP_CMT_WAN_ERROR_NOT		14
#define APSR_PPTP_CMT_SET_LINK_INFO		15

#define APSR_PPTP_DEFAULT_MAGIC_COOKIE 		0x1A2B3C4D

struct mod_pptp_header {
	unsigned short length;
	unsigned short message_type;
	unsigned int magic_cookie;
	unsigned short cmt;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
  	module_info.family      = MODULE_IDENT_FAMILY;
    	module_info.proto1      = MODULE_IDENT_PN1;
      	module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
	      
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_pptp_header *pptp_header;
  unsigned int magic_cookie;
  unsigned short message_type, cmt;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    pptp_header = (struct mod_pptp_header *)buf;
	    message_type = apsr_ntohs(pptp_header->message_type);
	    magic_cookie = apsr_ntohl(pptp_header->magic_cookie);
	    apsr_output("- PPTP Header -\n");
	    apsr_output("  Length: %u, Message Type: %d", apsr_ntohs(pptp_header->length), message_type);
	    if(verbose) 
		    apsr_output("%s", mod_pptp_messagetype[message_type]);
            apsr_output("\n  Magic-Cookie: 0x%08x", magic_cookie);
	    if(verbose && magic_cookie == APSR_PPTP_DEFAULT_MAGIC_COOKIE) 
		    apsr_output(" (default)");
	    if(message_type == APSR_PPTP_MESSAGE_TYPE_CONTROL) {
		    cmt = apsr_ntohl(pptp_header->cmt);
		    apsr_output("\n  Control Message Type: %d");
		    if(verbose) 
			    apsr_output("%s", mod_pptp_cmt[cmt]);
	    }
	    apsr_output("\n");	
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
