#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.02"
#define MODULE_NAME		"stun"
#define MODULE_DESCRIPTION	"Example Simple Traversal of UDP Through NAT protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_PN2	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	3478	
#define MODULE_IDENT_ID2	3478

#define MODULE_FILTER		"(tcp or udp) and port 3478"

#define APSR_MOD_STUN_TYPE_BIND_REQ	0x0001
#define APSR_MOD_STUN_TYPE_SHAR_SECREQ	0x0002
#define APSR_MOD_STUN_TYPE_BIND_RES	0x0101
#define APSR_MOD_STUN_TYPE_SHAR_SECRES	0x0102
#define APSR_MOD_STUN_TYPE_BIND_ERRRES	0x0111
#define APSR_MOD_STUN_TYPE_SHAR_ERRRES	0x0112

struct mod_stun_header {
	unsigned short type;
	unsigned short length;
	unsigned short trans_id;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = MODULE_IDENT_PN2;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;
extern unsigned short module_length;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_stun_header *stun_header;
  unsigned int i;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    stun_header = (struct mod_stun_header *)buf;
	    apsr_output("- STUN Header -\n");
	    apsr_output("  Type: %u", apsr_ntohs(stun_header->type));
	    if(verbose) {
		    switch(apsr_ntohs(stun_header->type)) {
			    case APSR_MOD_STUN_TYPE_BIND_REQ:
			      apsr_output(" (Bind Request)");
			      break;
			    case APSR_MOD_STUN_TYPE_SHAR_SECREQ:
			      apsr_output(" (Shared Secrect Request)");
			      break;
			    case APSR_MOD_STUN_TYPE_BIND_RES:
			      apsr_output(" (Bind Response)");
			      break;
			    case APSR_MOD_STUN_TYPE_SHAR_SECRES:
			      apsr_output(" (Shared Secrect Response)");
			      break;
			    case APSR_MOD_STUN_TYPE_BIND_ERRRES:
			      apsr_output(" (Bind Error Response)");
			      break;
			    case APSR_MOD_STUN_TYPE_SHAR_ERRRES:
			      apsr_output(" (Shared Secrect Error Response)");
			      break;
			    default:
			      apsr_output(" (Unknown)");
			      break;
		    }
            }
	    apsr_output("\n  Length: %u, Transaction ID: %u\n", 
			    apsr_ntohs(stun_header->length),
			    apsr_ntohs(stun_header->trans_id));

	    apsr_output("  Data: \"");
            for(i = 0; i <= module_length-6; i++) {
		    if(isprint(buf[i+6]))
			    apsr_output("%c", buf[i+6]);
                    else
                            apsr_output(".");
            }
            apsr_output("\"\n");
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
