#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.03"
#define MODULE_NAME		"trip"
#define MODULE_DESCRIPTION	"Example TRIP (Telephone Routing over IP) protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_ID1	6069	
#define MODULE_IDENT_ID2	6069

#define MODULE_FILTER		"tcp and port 6069"

#define APSR_TRIP_TYPE_OPEN		1	/* rfc 3219 */
#define APSR_TRIP_TYPE_UPDATE		2	/* rfc 3219 */
#define APSR_TRIP_TYPE_NOTI		3	/* rfc 3219 */
#define APSR_TRIP_TYPE_KEEP		4	/* rfc 3219 */

static const char *mod_trip_types[]= {
	" (Unknown)",
	" (Open)",
	" (Update)", 
	" (Notification)",
	" (Keep alive)",
};

struct mod_trip_header {
	unsigned short length;
	unsigned char type[1];
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
	module_info.proto1      = MODULE_IDENT_PN1;
	module_info.proto2      = 0;
	module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_trip_header *trip_header;
  unsigned short length, i;
  unsigned char *data;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    trip_header = (struct mod_trip_header *)buf;
	    length = apsr_ntohs(trip_header->length);
	    apsr_output("- TRIP Header -\n");
	    apsr_output("  Length: %u\n", length);
	    apsr_output("  Type: %u", trip_header->type[0]);
	    if(verbose) 
	    	  apsr_output("%s", mod_trip_types[trip_header->type[0]]);
	    /* If more than the header, than print data */
	    apsr_output("\n");
	    if(length > 3) {
              data = (buf+3);
              apsr_output("  Data: \"");
              for(i = 3; i <= length; i++) {
	                if(isprint(data[i]))
				apsr_output("%c", data[i]);
	                else
				apsr_output(".");
              }
              apsr_output("\"\n");
	    }
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
